﻿using System;
using wclCommon;
using wclWiFi;

namespace WiFiClient
{
    class Program
    {
        static void Main(string[] args)
        {
            wclMessageBroadcaster.SetSyncMethod(wclMessageSynchronizationKind.skApc);

            Console.Clear();

            Console.WriteLine("WIFI CLIENT DEMO APPLICATION");
            Console.WriteLine("============================");
            Console.WriteLine("This demo enumerates WiFi interfaces, networks and BBS");

            wclWiFiClient Client = new wclWiFiClient();
            Int32 Res = Client.Open();
            if (Res != wclErrors.WCL_E_SUCCESS)
                Console.WriteLine("Failed to open WiFi Direct client: 0x" + Res.ToString("X8"));
            else
            {
                wclWiFiInterfaceData[] Ifaces;
                Res = Client.EnumInterfaces(out Ifaces);
                if (Res != wclErrors.WCL_E_SUCCESS)
                    Console.WriteLine("Failed to enumerate WiFi interfaces: 0x" + Res.ToString("X8"));
                else
                {
                    foreach (wclWiFiInterfaceData IfaceData in Ifaces)
                    {
                        Guid Id = IfaceData.Id;

                        Console.WriteLine("Interface " + Id.ToString() + " found");
                        Console.WriteLine("  Description: " + IfaceData.Description);

                        Console.WriteLine("Try to enumerate available networks");
                        wclWiFiAvailableNetwork[] Networks;
                        Res = Client.EnumAvailableNetworks(Id, 0, out Networks);
                        if (Res != wclErrors.WCL_E_SUCCESS)
                            Console.WriteLine("      Failed to enumerate available networks: 0x" + Res.ToString("X8"));
                        else
                        {
                            foreach (wclWiFiAvailableNetwork Network in Networks)
                            {
                                // Ignore networks that has profile.
                                if (Network.ProfileName == "" && Network.Ssid != "")
                                {
                                    Console.WriteLine("      SSID: " + Network.Ssid);
                                    wclWiFiBss[] BssList;
                                    Res = Client.EnumBss(Id, Network.Ssid, wclWiFiBssType.bssInfrastructure, true, out BssList);
                                    if (Res != wclErrors.WCL_E_SUCCESS)
                                        Console.WriteLine("        Failed to enumerate BSS: 0x" + Res.ToString("X8"));
                                    else
                                    {
                                        if (BssList != null && BssList.Length > 0)
                                        {
                                            foreach (wclWiFiBss Bss in BssList)
                                            {
                                                Console.WriteLine("        BSS: " + Bss.Ssid);
                                                Console.WriteLine("          Frequency: " + Bss.ChCenterFrequency.ToString() + "; MAC: " +
                                                    Bss.Mac + "; RSSI: " + Bss.Rssi.ToString());
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
                Client.Close();
            }
            Console.WriteLine("Press ENTER to exit");
            Console.ReadLine();
        }
    }
}
