﻿using System;
using System.Threading;

using wclCommon;
using wclWiFi;

namespace WiFiDirectSoftAP
{
    class Program
    {
        private class SoftAp
        {
            private ManualResetEvent TerminateEvent;
            private Thread Thread;

            private void SoftApStopped(Object Sender, EventArgs e)
            {
                Console.WriteLine("WiFi Direct Soft AP has been stopped");
            }

            private void SoftApDeviceConnected(Object Sender, wclWiFiDirectDevice Device)
            {
                Console.WriteLine("Device has been connected");
                Console.WriteLine("  Name: " + Device.Name);
                Console.WriteLine("  Local IP addresss: " + Device.LocalAddress);
                Console.WriteLine("  Remote IP addresss: " + Device.RemoteAddress);
            }

            private void SoftApDeviceDisconnected(Object Sender, wclWiFiDirectDevice Device)
            {
                Console.WriteLine("Device " + Device.Name + " has been disconnected");
            }

            private void SoftApStarted(Object Sender, EventArgs e)
            {
                Console.WriteLine("WiFi Direct Soft AP has been started");

                String ts;
                Int32 Res = ((wclWiFiSoftAP)Sender).GetSsid(out ts);
                if (Res != wclErrors.WCL_E_SUCCESS)
                    Console.WriteLine("  SSID: read error 0x" + Res.ToString("X8"));
                else
                    Console.WriteLine("  SSID: " + ts);

                Res = ((wclWiFiSoftAP)Sender).GetPassphrase(out ts);
                if (Res != wclErrors.WCL_E_SUCCESS)
                    Console.WriteLine("  Passphrase: read error 0x" + Res.ToString("X8"));
                else
                    Console.WriteLine("  Passphrase: " + ts);
            }

            private void ThreadProc()
            {
                wclWiFiSoftAP Ap = new wclWiFiSoftAP();
                Ap.OnStarted += SoftApStarted;
                Ap.OnStopped += SoftApStopped;
                Ap.OnDeviceConnected += SoftApDeviceConnected;
                Ap.OnDeviceDisconnected += SoftApDeviceDisconnected;

                Int32 Res = Ap.Start("WiFi_Framework", "12345678");
                if (Res != wclErrors.WCL_E_SUCCESS)
                    Console.WriteLine("Failed to start WiFi Direct Soft AP: 0x" + Res.ToString("X8"));

                TerminateEvent.WaitOne();

                if (Ap.Active)
                {
                    Res = Ap.Stop();
                    if (Res != wclErrors.WCL_E_SUCCESS)
                        Console.WriteLine("Failed to stop WiFi Direct Soft AP: 0x" + Res.ToString("X8"));
                }
                Ap = null;
            }

            public void Run()
            {
                TerminateEvent = new ManualResetEvent(false);
                Thread = new Thread(ThreadProc);
                Thread.Start();
            }

            public void Stop()
            {
                TerminateEvent.Set();
                Thread.Join();
                TerminateEvent.Close();
            }
        };
        
        static void Main(string[] args)
        {
            wclMessageBroadcaster.SetSyncMethod(wclMessageSynchronizationKind.skApc);

            Console.Clear();
            Console.WriteLine("WIFI DIRECT SOFT AP DEMO");
            Console.WriteLine("========================");
            SoftAp Ap = new SoftAp();
            Ap.Run();
            Console.WriteLine("PRESS ANY KEY TO STOP WIFI DIRECT SOFT AP");
            Console.ReadKey();
            Ap.Stop();
        }
    };
}
