﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using wclCommon;
using wclWiFi;

namespace WiFiDirectClient
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private wclWiFiDirectDeviceWatcher Watcher;
        private wclWiFiDirectClient Client;

        internal class DeviceItem
        {
            public String Name { get; set; }
            public String Id { get; set; }
        };

        public MainWindow()
        {
            InitializeComponent();
        }

        private void fmMain_Loaded(object sender, RoutedEventArgs e)
        {
            cbPairingProcedure.SelectedIndex = 0;

            Watcher = new wclWiFiDirectDeviceWatcher();
            Watcher.OnDeviceFound += new wclWiFiDirectDeviceFoundEvent(Watcher_OnDeviceFound);
            Watcher.OnDiscoveringCompleted += new wclWiFiDirectDiscoveringCompletedEvent(Watcher_OnDiscoveringCompleted);
            Watcher.OnDiscoveringStarted += new EventHandler(Watcher_OnDiscoveringStarted);

            Client = new wclWiFiDirectClient();
            Client.OnPairCompleted += new wclWiFiDirectPairCompletedEvent(Client_OnPairCompleted);
            Client.OnPairConfirm += new wclWiFiDirectPairConfirmEvent(Client_OnPairConfirm);
            Client.OnPairDisplayPin += new wclWiFiDirectPairDisplayPinEvent(Client_OnPairDisplayPin);
            Client.OnPairGetParams += new wclWiFiDirectPairGetParamsEvent(Client_OnPairGetParams);
            Client.OnPairProvidePin += new wclWiFiDirectPairProvidePinEvent(Client_OnPairProvidePin);
            Client.OnDeviceConnected += new wclWiFiDirectConnectedEvent(Client_OnDeviceConnected);
            Client.OnDeviceDisconnected += new wclWiFiDirectDisconnectedEvent(Client_OnDeviceDisconnected);
        }

        void Watcher_OnDeviceFound(object Sender, string id, string name)
        {
            lvDevices.Items.Add(new DeviceItem
            {
                Name = name,
                Id = id
            });

            lbLog.Items.Add("Device found");
        }

        void Watcher_OnDiscoveringCompleted(object Sender, int Error)
        {
            lbLog.Items.Add("Discovering completed with result: 0x" + Error.ToString("X8"));
        }

        void Watcher_OnDiscoveringStarted(object sender, EventArgs e)
        {
            lvDevices.Items.Clear();
            lbLog.Items.Add("Discovering started");
        }

        void Client_OnPairCompleted(object Sender, wclWiFiDirectDevice Device, int Result)
        {
            lbLog.Items.Add("Pairing completed with result: 0x" + Result.ToString("X8"));
        }

        void Client_OnPairConfirm(object Sender, wclWiFiDirectDevice Device, out bool Confirm)
        {
            Confirm = MessageBox.Show("Confirm pairing", "Pairing", MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.Yes;
        }

        void Client_OnPairDisplayPin(object Sender, wclWiFiDirectDevice Device, string Pin)
        {
            MessageBox.Show("Use PIN: " + Pin);
        }

        void Client_OnPairGetParams(object Sender, wclWiFiDirectDevice Device, out short GroupOwnerIntent, out wclWiFiDirectConfigurationMethod ConfigurationMethods, out wclWiFiDirectPairingProcedure PairingProcedure)
        {
            GroupOwnerIntent = Convert.ToInt16(edGroupOwnerIntent.Text);

            ConfigurationMethods = 0;
            if (cbDisplayPIN.IsChecked == true)
                ConfigurationMethods |= wclWiFiDirectConfigurationMethod.cmDisplayPin;
            if (cbProvidePIN.IsChecked == true)
                ConfigurationMethods |= wclWiFiDirectConfigurationMethod.cmProvidePin;
            if (cbPushButton.IsChecked == true)
                ConfigurationMethods |= wclWiFiDirectConfigurationMethod.cmPushButton;

            if (cbPairingProcedure.SelectedIndex == 0)
                PairingProcedure = wclWiFiDirectPairingProcedure.ppInvitation;
            else
                PairingProcedure = wclWiFiDirectPairingProcedure.ppGroupOwnerNegotiation;
        }

        void Client_OnPairProvidePin(object Sender, wclWiFiDirectDevice Device, out string Pin)
        {
            Pin = Microsoft.VisualBasic.Interaction.InputBox("Enter PIN for ", "Pairing", "");
        }

        void Client_OnDeviceConnected(object Sender, wclWiFiDirectDevice Device, int Error)
        {
            lbLog.Items.Add("Connected with result: 0x" + Error.ToString("X8"));
            if (Error == wclErrors.WCL_E_SUCCESS)
            {
                lbLog.Items.Add("  Local IP: " + Client.LocalAddress);
                lbLog.Items.Add("  Remote IP: " + Client.RemoteAddress);
            }
        }

        void Client_OnDeviceDisconnected(object Sender, wclWiFiDirectDevice Device, int Reason)
        {
            lbLog.Items.Add("Disconnected. Reason: 0x" + Reason.ToString("X8"));
        }

        private void btDiscover_Click(object sender, RoutedEventArgs e)
        {
            Int32 Res = Watcher.Discover();
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Discover error: 0x" + Res.ToString("X8"));
        }

        private void btTerminate_Click(object sender, RoutedEventArgs e)
        {
            Int32 Res = Watcher.Terminate();
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Terminate error: 0x" + Res.ToString("X8"));
        }

        private void fmMain_Closed(object sender, EventArgs e)
        {
            Client.Disconnect();
            Watcher.Terminate();
        }

        private void btClear_Click(object sender, RoutedEventArgs e)
        {
            lbLog.Items.Clear();
        }

        private void btDisconnect_Click(object sender, RoutedEventArgs e)
        {
            Int32 Res = Client.Disconnect();
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Disconnect error: 0x" + Res.ToString("X8"));
        }

        private void btConnect_Click(object sender, RoutedEventArgs e)
        {
            if (lvDevices.SelectedItems.Count == 0)
                MessageBox.Show("Select device");
            else
            {
                Int32 Res = Client.Connect(((DeviceItem)lvDevices.SelectedItems[0]).Id);
                if (Res != wclErrors.WCL_E_SUCCESS)
                    MessageBox.Show("Connect Error: 0x" + Res.ToString("X8"));
            }
        }

        private void btIsPaired_Click(object sender, RoutedEventArgs e)
        {
            if (lvDevices.SelectedItems.Count == 0)
                MessageBox.Show("Select device");
            else
            {
                Boolean IsPaired;
                Int32 Res = Watcher.IsPaired(((DeviceItem)lvDevices.SelectedItems[0]).Id, out IsPaired);
                if (Res != wclErrors.WCL_E_SUCCESS)
                    MessageBox.Show("Get paired error: 0x" + Res.ToString("X8"));
                else
                {
                    if (IsPaired)
                        MessageBox.Show("Device is paired");
                    else
                        MessageBox.Show("Device is NOT paired");
                }
            }
        }

        private void btUnpair_Click(object sender, RoutedEventArgs e)
        {
            if (lvDevices.SelectedItems.Count == 0)
                MessageBox.Show("Select device");
            else
            {
                Int32 Res = Watcher.Unpair(((DeviceItem)lvDevices.SelectedItems[0]).Id);
                if (Res != wclErrors.WCL_E_SUCCESS)
                    MessageBox.Show("Unpair error: 0x" + Res.ToString("X8"));
                else
                    MessageBox.Show("Unpaired");
            }
        }

        private void btGetName_Click(object sender, RoutedEventArgs e)
        {
            if (lvDevices.SelectedItems.Count == 0)
                MessageBox.Show("Select device");
            else
            {
                String Name;
                Int32 Res = Watcher.GetName(((DeviceItem)lvDevices.SelectedItems[0]).Id, out Name);
                if (Res != wclErrors.WCL_E_SUCCESS)
                    MessageBox.Show("Get name error: 0x" + Res.ToString("X8"));
                else
                    MessageBox.Show("Device name: " + Name);
            }
        }

        private void btEnumPaired_Click(object sender, RoutedEventArgs e)
        {
            List<String> Paired = new List<String>();
            Int32 Res = Watcher.EnumPairedDevices(Paired);
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Enum paired error: 0x" + Res.ToString("X8"));
            else
            {
                lvDevices.Items.Clear();

                if (Paired.Count > 0)
                {
                    foreach (String id in Paired)
                    {
                        String name;
                        Res = Watcher.GetName(id, out name);
                        if (Res != wclErrors.WCL_E_SUCCESS)
                            name = "Error: 0x" + Res.ToString("X8");

                        lvDevices.Items.Add(new DeviceItem
                        {
                            Name = name,
                            Id = id
                        });
                    }
                }
            }
        }
    }
}
