﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using wclWiFi;
using wclCommon;

namespace WiFiHostedNetwork
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private wclWiFiHostedNetwork Network;

        internal class DataItem
        {
            public String ParamName { get; set; }
            public String ParamValue { get; set; }
        };

        public MainWindow()
        {
            InitializeComponent();
        }

        private void fmMain_Loaded(object sender, RoutedEventArgs e)
        {
            Network = new wclWiFiHostedNetwork();
            Network.AfterOpen += new EventHandler(Network_AfterOpen);
            Network.BeforeClose += new EventHandler(Network_BeforeClose);
        }

        void Network_AfterOpen(object sender, EventArgs e)
        {
            MessageBox.Show("WiFi Hosted Network opened");
        }

        void Network_BeforeClose(object sender, EventArgs e)
        {
            MessageBox.Show("WiFi Hosted Network closing");
        }

        private void btOpen_Click(object sender, RoutedEventArgs e)
        {
            ShowResult(Network.Open());
        }

        private Boolean ShowResult(Int32 Res)
        {
            if (Res != wclErrors.WCL_E_SUCCESS)
            {
                MessageBox.Show("Error: 0x" + Res.ToString("X8"));
                return false;
            }

            return true;
        }

        private void btClose_Click(object sender, RoutedEventArgs e)
        {
            ShowResult(Network.Close());
        }

        private void btHnGetConnectionSettings_Click(object sender, RoutedEventArgs e)
        {
            wclWiFiHostedNetworkConnectionSettings Settings;

            HnClear();

            if (ShowResult(Network.GetConnectionSettings(out Settings)))
            {
                HnAddData("SSID", Settings.Ssid);
                HnAddData("Max. number of peers", Settings.MaxNumberOfPeers.ToString());
            }
        }

        private void HnClear()
        {
            lvHnData.Items.Clear();
        }

        private void btClear_Click(object sender, RoutedEventArgs e)
        {
            HnClear();
        }

        private void HnAddData(String paramName, String paramValue)
        {
            lvHnData.Items.Add(new DataItem
            {
                ParamName = paramName,
                ParamValue = paramValue
            });
        }

        private void btHnGetKey_Click(object sender, RoutedEventArgs e)
        {
            HnClear();

            Byte[] KeyData;
            Boolean IsPassPhrase;
            Boolean Persistent;
            if (ShowResult(Network.GetKey(out KeyData, out IsPassPhrase, out Persistent)))
            {
                String Str;
                if (IsPassPhrase)
                    Str = System.Text.Encoding.ASCII.GetString(KeyData);
                else
                {
                    Str = "";
                    if (KeyData != null)
                    {
                        for (Int32 i = 0; i < KeyData.Length; i++)
                            Str = Str + KeyData[i].ToString("X2");
                    }
                }

                if (KeyData != null)
                    HnAddData("Key length", KeyData.Length.ToString());
                else
                    HnAddData("Key length", "0");
                HnAddData("Key data", Str);
                HnAddData("Is pass phrase", IsPassPhrase.ToString());
                HnAddData("Persistent", Persistent.ToString());
            }
        }

        private void btHnGetProfile_Click(object sender, RoutedEventArgs e)
        {
            HnClear();

            String Xml;
            if (ShowResult(Network.GetProfile(out Xml)))
                HnAddData("Profile XML", Xml);
        }

        private void btHnGetSecuritySettings_Click(object sender, RoutedEventArgs e)
        {
            HnClear();

            wclWiFiAuthCipherPair Settings;
            if (ShowResult(Network.GetSecuritySettings(out Settings)))
            {
                HnAddData("Auth algorithm", Settings.AuthAlgorithm.ToString());
                HnAddData("Cipher algorithm", Settings.CipherAlgorithm.ToString());
            }
        }

        private void btHnGetStatus_Click(object sender, RoutedEventArgs e)
        {
            HnClear();

            wclWiFiHostedNetworkStatus Status;
            if (ShowResult(Network.GetStatus(out Status)))
            {
                HnAddData("State", Status.State.ToString());
                HnAddData("Interface ID", Status.Id.ToString());
                HnAddData("BSS ID", Status.BssId);
                HnAddData("Phy", Status.Phy.ToString());
                HnAddData("ChannelFrequency", Status.ChannelFrequency.ToString());

                String Address;
                Int32 Res = Network.GetLocalIp(out Address);
                if (Res == wclErrors.WCL_E_SUCCESS)
                    HnAddData("Local IP", Address);
                else
                    HnAddData("Local IP", "Get failed: 0x" + Res.ToString("X8"));

                if (Status.Peers != null)
                {
                    for (Int32 i = 0; i < Status.Peers.Length; i++)
                    {
                        HnAddData("Peer[" + i.ToString() + "] MAC", Status.Peers[i].Mac);
                        Res = Network.GetRemoteIp(Status.Peers[i].Mac, out Address);
                        if (Res == wclErrors.WCL_E_SUCCESS)
                            HnAddData("Peer[" + i.ToString() + "] IP", Address);
                        else
                            HnAddData("Peer[" + i.ToString() + "] IP", "Get failed: 0x" + Res.ToString("X8"));
                        HnAddData("Peer[" + i.ToString() + "] auth state", Status.Peers[i].AuthState.ToString());
                    }
                }
            }
        }

        private void btHnRefreshSecuritySettings_Click(object sender, RoutedEventArgs e)
        {
            ShowResult(Network.RefreshSecuritySettings());
        }

        private void btHnSetSettings_Click(object sender, RoutedEventArgs e)
        {
            wclWiFiHostedNetworkConnectionSettings Settings = new wclWiFiHostedNetworkConnectionSettings();
            Settings.Ssid = edSsHnid.Text;
            Settings.MaxNumberOfPeers = Convert.ToUInt16(edHnNumberOfPeers.Text);
            ShowResult(Network.SetConnectionSettings(Settings));
        }

        private void btHnSetKey_Click(object sender, RoutedEventArgs e)
        {
            Byte[] Tmp = System.Text.Encoding.ASCII.GetBytes(edHnKey.Text);
            Byte[] KeyData = new Byte[Tmp.Length + 1];
            Tmp.CopyTo(KeyData, 0);
            ShowResult(Network.SetKey(KeyData, true, cbHnPersistent.IsChecked == true));
        }

        private void btEnable_Click(object sender, RoutedEventArgs e)
        {
            ShowResult(Network.SetState(true));
        }

        private void btDisable_Click(object sender, RoutedEventArgs e)
        {
            ShowResult(Network.SetState(false));
        }

        private void fmMain_Closed(object sender, EventArgs e)
        {
            Network.Close();
        }

        private void btHnGetState_Click(object sender, RoutedEventArgs e)
        {
            HnClear();

            Boolean State;
            if (ShowResult(Network.GetState(out State)))
                HnAddData("Enabled", State.ToString());
        }

        private void btHnRestart_Click(object sender, RoutedEventArgs e)
        {
            ShowResult(Network.Restart());
        }
    }
}
