using System;
using System.Windows.Forms;

using wclCommon;
using wclWiFi;

namespace NetworkListManager
{
    public partial class main : Form
    {
        private wclNetworkListManager wclNetworkListManager;

        public main()
        {
            InitializeComponent();
        }

        private void main_Load(object sender, EventArgs e)
        {
            cbNetworkType.SelectedIndex = 2;
            cbCategory.SelectedIndex = 0;

            wclNetworkListManager = new wclNetworkListManager();
            wclNetworkListManager.AfterOpen += new EventHandler(wclNetworkListManager_AfterOpen);
            wclNetworkListManager.BeforeClose += new EventHandler(wclNetworkListManager_BeforeClose);
            wclNetworkListManager.OnConnectivityChanged += new wclNlmConnectivityChangeEvent(wclNetworkListManager_OnConnectivityChanged);
            wclNetworkListManager.OnNetworkAdded += new wclNlmNetworkEvent(wclNetworkListManager_OnNetworkAdded);
            wclNetworkListManager.OnNetworkConnectivityChanged += new wclNlmNetworkConnectivityChangedEvent(wclNetworkListManager_OnNetworkConnectivityChanged);
            wclNetworkListManager.OnNetworkDeleted += new wclNlmNetworkEvent(wclNetworkListManager_OnNetworkDeleted);
            wclNetworkListManager.OnNetworkPropertyChanged += new wclNlmNetworkPropertyChangedEvent(wclNetworkListManager_OnNetworkPropertyChanged);
            wclNetworkListManager.OnConnectionConnectivityChanged += new wclNlmConnectionConnectivityChangedEvent(wclNetworkListManager_OnConnectionConnectivityChanged);
            wclNetworkListManager.OnConnectionPropertyChanged += new wclNlmConnectionPropertyChangedEvent(wclNetworkListManager_OnConnectionPropertyChanged);
        }

        void wclNetworkListManager_OnConnectionPropertyChanged(object Sender, Guid ConnectionId, wclNlmConnectionProperty Prop)
        {
            lbEvents.Items.Add("Connection: " + ConnectionId.ToString() + " property changed: " + Prop.ToString());
        }

        void wclNetworkListManager_OnConnectionConnectivityChanged(object Sender, Guid ConnectionId, wclNlmConnectivity Connectivity)
        {
            lbEvents.Items.Add("Connection: " + ConnectionId.ToString() + " connectivity changed: " + Connectivity.ToString());
        }

        void wclNetworkListManager_OnNetworkPropertyChanged(object Sender, Guid NetworkId, wclNlmNetworkPropertyChange Change)
        {
            lbEvents.Items.Add("Network " + NetworkId.ToString() + " property changed: " + Change.ToString());
        }

        void wclNetworkListManager_OnNetworkDeleted(object Sender, Guid NetworkId)
        {
            lbEvents.Items.Add("Network deleted: " + NetworkId.ToString());
        }

        void wclNetworkListManager_OnNetworkConnectivityChanged(object Sender, Guid NetworkId, wclNlmConnectivity Connectivity)
        {
            lbEvents.Items.Add("Network " + NetworkId.ToString() + " connectivity changed: " + Connectivity.ToString());
        }

        void wclNetworkListManager_OnNetworkAdded(object Sender, Guid NetworkId)
        {
            lbEvents.Items.Add("Network added: " + NetworkId.ToString());
        }

        void wclNetworkListManager_OnConnectivityChanged(object Sender, wclNlmConnectivity Connectivity)
        {
            lbEvents.Items.Add("Connectivity changed: " + Connectivity.ToString());
        }

        void wclNetworkListManager_BeforeClose(object sender, EventArgs e)
        {
            lbEvents.Items.Add("BeforeClose");
        }

        void wclNetworkListManager_AfterOpen(object sender, EventArgs e)
        {
            lbEvents.Items.Add("AfterOpen");
        }

        private void main_FormClosed(object sender, FormClosedEventArgs e)
        {
            wclNetworkListManager.Close();
        }

        private void btClear_Click(object sender, EventArgs e)
        {
            lbEvents.Items.Clear();
        }

        private void btOpen_Click(object sender, EventArgs e)
        {
            ShowError(wclNetworkListManager.Open());
        }

        private void btClose_Click(object sender, EventArgs e)
        {
            ShowError(wclNetworkListManager.Close());
        }

        private Boolean ShowError(Int32 Error)
        {
            if (Error != wclErrors.WCL_E_SUCCESS)
            {
                MessageBox.Show("Error: 0x" + Error.ToString("X8"));
                return true;
            }

            return false;
        }

        private void btIsConnected_Click(object sender, EventArgs e)
        {
            bool Connected;
            if (!ShowError(wclNetworkListManager.GetConnected(out Connected)))
            {
                if (Connected)
                    MessageBox.Show("Connected");
                else
                    MessageBox.Show("NOT Connected");
            }
        }

        private void btIsConnectedToInternet_Click(object sender, EventArgs e)
        {
            bool Connected;
            if (!ShowError(wclNetworkListManager.GetConnectedToInternet(out Connected)))
            {
                if (Connected)
                    MessageBox.Show("Connected");
                else
                    MessageBox.Show("NOT Connected");
            }
        }

        private void btConnectivity_Click(object sender, EventArgs e)
        {
            wclNlmConnectivity Connectivity;
            if (!ShowError(wclNetworkListManager.GetConnectivity(out Connectivity)))
                MessageBox.Show("Connectivity: " + Connectivity.ToString());
        }

        private void btEnumNetworks_Click(object sender, EventArgs e)
        {
            lvNetworks.Items.Clear();

            wclNlmEnumNetwork Flag = (wclNlmEnumNetwork)cbNetworkType.SelectedIndex;
            wclNlmNetwork[] Networks;
            Int32 Res = wclNetworkListManager.GetNetworks(Flag, out Networks);
            try
            {
                if (!ShowError(Res))
                {
                    for (int i = 0; i < Networks.Length; i++)
                    {
                        wclNlmNetwork Network = Networks[i];
                        ListViewItem Item = lvNetworks.Items.Add("");

                        Guid Id;
                        Res = Network.GetNetworkId(out Id);
                        if (Res == wclErrors.WCL_E_SUCCESS)
                            Item.Text = Id.ToString();
                        else
                            Item.Text = "Error: 0x" + Res.ToString("X8");

                        wclNlmNetworkCategory Category;
                        Res = Network.GetCategory(out Category);
                        if (Res == wclErrors.WCL_E_SUCCESS)
                            Item.SubItems.Add(Category.ToString());
                        else
                            Item.SubItems.Add("Error: 0x" + Res.ToString("X8"));

                        wclNlmConnectivity Connectivity;
                        Res = Network.GetConnectivity(out Connectivity);
                        if (Res == wclErrors.WCL_E_SUCCESS)
                            Item.SubItems.Add(Connectivity.ToString());
                        else
                            Item.SubItems.Add("Error: 0x" + Res.ToString("X8"));

                        String Description;
                        Res = Network.GetDescription(out Description);
                        if (Res == wclErrors.WCL_E_SUCCESS)
                            Item.SubItems.Add(Description);
                        else
                            Item.SubItems.Add("Error: 0x" + Res.ToString("X8"));

                        wclNlmDomainType Domain;
                        Res = Network.GetDomainType(out Domain);
                        if (Res == wclErrors.WCL_E_SUCCESS)
                            Item.SubItems.Add(Domain.ToString());
                        else
                            Item.SubItems.Add("Error: 0x" + Res.ToString("X8"));

                        String AName;
                        Res = Network.GetName(out AName);
                        if (Res == wclErrors.WCL_E_SUCCESS)
                            Item.SubItems.Add(AName);
                        else
                            Item.SubItems.Add("Error: 0x" + Res.ToString("X8"));

                        DateTime CreationDateTime;
                        DateTime ConnectedDateTime;
                        Res = Network.GetTimeCreatedAndConnected(out CreationDateTime,
                            out ConnectedDateTime);
                        if (Res == wclErrors.WCL_E_SUCCESS)
                        {
                            Item.SubItems.Add(CreationDateTime.ToString());
                            Item.SubItems.Add(ConnectedDateTime.ToString());
                        }
                        else
                        {
                            Item.SubItems.Add("Error: 0x" + Res.ToString("X8"));
                            Item.SubItems.Add("Error: 0x" + Res.ToString("X8"));
                        }

                        bool Connected;
                        Res = Network.GetConnected(out Connected);
                        if (Res == wclErrors.WCL_E_SUCCESS)
                            Item.SubItems.Add(Connected.ToString());
                        else
                            Item.SubItems.Add("Error: 0x" + Res.ToString("X8"));

                        Res = Network.GetConnectedToInternet(out Connected);
                        if (Res == wclErrors.WCL_E_SUCCESS)
                            Item.SubItems.Add(Connected.ToString());
                        else
                            Item.SubItems.Add("Error: 0x" + Res.ToString("X8"));
                    }
                }
            }
            finally
            {
                Networks = null;
            }
        }

        private void btRefresh_Click(object sender, EventArgs e)
        {
            if (lvNetworks.SelectedItems.Count == 0)
            {
                MessageBox.Show("Select network");
                return;
            }

            wclNlmNetwork Network;
            ListViewItem Item = lvNetworks.SelectedItems[0];
            Guid Id = new Guid(Item.SubItems[0].Text);
            if (!ShowError(wclNetworkListManager.GetNetwork(Id, out Network)))
            {
                try
                {
                    wclNlmNetworkCategory Category;
                    int Res = Network.GetCategory(out Category);
                    if (Res == wclErrors.WCL_E_SUCCESS)
                        Item.SubItems[1].Text = Category.ToString();
                    else
                        Item.SubItems[1].Text = "Error: 0x" + Res.ToString("X8");

                    wclNlmConnectivity Connectivity;
                    Res = Network.GetConnectivity(out Connectivity);
                    if (Res == wclErrors.WCL_E_SUCCESS)
                        Item.SubItems[2].Text = Connectivity.ToString();
                    else
                        Item.SubItems[2].Text = "Error: 0x" + Res.ToString("X8");

                    String Description;
                    Res = Network.GetDescription(out Description);
                    if (Res == wclErrors.WCL_E_SUCCESS)
                        Item.SubItems[3].Text = Description;
                    else
                        Item.SubItems[3].Text = "Error: 0x" + Res.ToString("X8");

                    wclNlmDomainType Domain;
                    Res = Network.GetDomainType(out Domain);
                    if (Res == wclErrors.WCL_E_SUCCESS)
                        Item.SubItems[4].Text = Domain.ToString();
                    else
                        Item.SubItems[4].Text = "Error: 0x" + Res.ToString("X8");

                    String AName;
                    Res = Network.GetName(out AName);
                    if (Res == wclErrors.WCL_E_SUCCESS)
                        Item.SubItems[5].Text = AName;
                    else
                        Item.SubItems[5].Text = "Error: 0x" + Res.ToString("X8");

                    DateTime CreationDateTime;
                    DateTime ConnectedDateTime;
                    Res = Network.GetTimeCreatedAndConnected(out CreationDateTime,
                        out ConnectedDateTime);
                    if (Res == wclErrors.WCL_E_SUCCESS)
                    {
                        Item.SubItems[6].Text = CreationDateTime.ToString();
                        Item.SubItems[7].Text = ConnectedDateTime.ToString();
                    }
                    else
                    {
                        Item.SubItems[6].Text = "Error: 0x" + Res.ToString("X8");
                        Item.SubItems[7].Text = "Error: 0x" + Res.ToString("X8");
                    }

                    bool Connected;
                    Res = Network.GetConnected(out Connected);
                    if (Res == wclErrors.WCL_E_SUCCESS)
                        Item.SubItems[8].Text = Connected.ToString();
                    else
                        Item.SubItems[8].Text = "Error: 0x" + Res.ToString("X8");

                    Res = Network.GetConnectedToInternet(out Connected);
                    if (Res == wclErrors.WCL_E_SUCCESS)
                        Item.SubItems[9].Text = Connected.ToString();
                    else
                        Item.SubItems[9].Text = "Error: 0x" + Res.ToString("X8");
                }
                finally
                {
                    Network = null;
                }
            }
        }

        private void btSetName_Click(object sender, EventArgs e)
        {
            if (lvNetworks.SelectedItems.Count == 0)
            {
                MessageBox.Show("Select network");
                return;
            }

            wclNlmNetwork Network;
            int Res = wclNetworkListManager.GetNetwork(new Guid(lvNetworks.SelectedItems[0].SubItems[0].Text),
                out Network);
            if (!ShowError(Res))
            {
                if (ShowError(Network.SetName(edNewNameOrDescription.Text)))
                    MessageBox.Show("Admin privilegies required");

                Network = null;
            }
        }

        private void btSetDescription_Click(object sender, EventArgs e)
        {
            if (lvNetworks.SelectedItems.Count == 0)
            {
                MessageBox.Show("Select network");
                return;
            }

            wclNlmNetwork Network;
            int Res = wclNetworkListManager.GetNetwork(new Guid(lvNetworks.SelectedItems[0].SubItems[0].Text),
                out Network);
            if (!ShowError(Res))
            {
                if (ShowError(Network.SetDescription(edNewNameOrDescription.Text)))
                    MessageBox.Show("Admin privilegies required");

                Network = null;
            }
        }

        private void btSetCategory_Click(object sender, EventArgs e)
        {
            if (lvNetworks.SelectedItems.Count == 0)
            {
                MessageBox.Show("Select network");
                return;
            }

            wclNlmNetwork Network;
            int Res = wclNetworkListManager.GetNetwork(new Guid(lvNetworks.SelectedItems[0].SubItems[0].Text),
                out Network);
            if (!ShowError(Res))
            {
                if (ShowError(Network.SetCategory((wclNlmNetworkCategory)cbCategory.SelectedIndex)))
                    MessageBox.Show("Admin privilegies required");

                Network = null;
            }
        }

        private void ShowConnections(ref wclNlmConnection[] Connections)
        {
            lvConnections.Items.Clear();
            
            if (Connections == null)
                return;
            
            try
            {
                for (int i = 0; i < Connections.Length; i++)
                {
                    wclNlmConnection Connection = Connections[i];
                    ListViewItem Item = lvConnections.Items.Add("");
                    
                    Guid Id;
                    int Res = Connection.GetAdapterId(out Id);
                    if (Res == wclErrors.WCL_E_SUCCESS)
                        Item.Text = Id.ToString();
                    else
                        Item.Text = "Error: 0x" + Res.ToString("X8");
                    
                    bool Connected;
                    Res = Connection.GetConnected(out Connected);
                    if (Res == wclErrors.WCL_E_SUCCESS)
                        Item.SubItems.Add(Connected.ToString());
                    else
                        Item.SubItems.Add("Error: 0x" + Res.ToString("X8"));
                    
                    Res = Connection.GetConnectedToInternet(out Connected);
                    if (Res == wclErrors.WCL_E_SUCCESS)
                        Item.SubItems.Add(Connected.ToString());
                    else
                        Item.SubItems.Add("Error: 0x" + Res.ToString("X8"));
                    
                    wclNlmConnectivity Connectivity;
                    Res = Connection.GetConnectivity(out Connectivity);
                    if (Res == wclErrors.WCL_E_SUCCESS)
                        Item.SubItems.Add(Connectivity.ToString());
                    else
                        Item.SubItems.Add("Error: 0x" + Res.ToString("X8"));
            
                    wclNlmDomainType Domain;
                    Res = Connection.GetDomainType(out Domain);
                    if (Res == wclErrors.WCL_E_SUCCESS)
                        Item.SubItems.Add(Domain.ToString());
                    else
                        Item.SubItems.Add("Error: 0x" + Res.ToString("X8"));
                    
                    Res = Connection.GetId(out Id);
                    if (Res == wclErrors.WCL_E_SUCCESS)
                        Item.SubItems.Add(Id.ToString());
                    else
                        Item.SubItems.Add("Error: 0x" + Res.ToString("X8"));
                }
            }
            finally
            {
                Connections = null;
            }
        }

        private void btGetMachineConnections_Click(object sender, EventArgs e)
        {
            wclNlmConnection[] Connections;
            if (!ShowError(wclNetworkListManager.GetConnections(out Connections)))
                ShowConnections(ref Connections);
        }

        private void btGetNetworkConnections_Click(object sender, EventArgs e)
        {
            if (lvNetworks.SelectedItems.Count == 0)
            {
                MessageBox.Show("Select network");
                return;
            }

            wclNlmNetwork Network;
            int Res = wclNetworkListManager.GetNetwork(new Guid(lvNetworks.SelectedItems[0].SubItems[0].Text),
                out Network);
            if (!ShowError(Res))
            {
                try
                {
                    wclNlmConnection[] Connections;
                    if (!ShowError(Network.GetConnections(out Connections)))
                        ShowConnections(ref Connections);
                }
                finally
                {
                    Network = null;
                }
            }
        }

        private void btDelete_Click(object sender, EventArgs e)
        {
            if (lvNetworks.SelectedItems.Count == 0)
            {
                MessageBox.Show("Select network");
                return;
            }

            int Res = wclNetworkListManager.DeleteNetwork(new Guid(lvNetworks.SelectedItems[0].SubItems[0].Text));
            if (!ShowError(Res))
                lvNetworks.Items.Remove(lvNetworks.SelectedItems[0]);
        }
    }
}