using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using wclWiFi;
using wclCommon;

namespace WiFiDirectSoftAP
{
    public partial class fmMain : Form
    {
        private wclWiFiSoftAP wclSoftAP;

        public fmMain()
        {
            InitializeComponent();
        }

        private void fmMain_Load(object sender, EventArgs e)
        {
            wclSoftAP = new wclWiFi.wclWiFiSoftAP();
            wclSoftAP.OnStarted += new EventHandler(wclSoftAP_OnStarted);
            wclSoftAP.OnStopped += new EventHandler(wclSoftAP_OnStopped);
            wclSoftAP.OnDeviceConnected += new wclWiFiDirectSoftAPConnectedEvent(wclSoftAP_OnDeviceConnected);
            wclSoftAP.OnDeviceConnectError += new wclWiFiDirectSoftAPConnectErrorEvent(wclSoftAP_OnDeviceConnectError);
            wclSoftAP.OnDeviceDisconnected += new wclWiFiDirectSoftAPDisconnectedEvent(wclSoftAP_OnDeviceDisconnected);
            wclSoftAP.OnDeviceAccept += new wclWiFiDirectSoftAPAcceptEvent(wclSoftAP_OnDeviceAccept);
        }

        void wclSoftAP_OnDeviceAccept(object Sender, wclWiFiDirectDevice Device, out bool Accept)
        {
            Accept = (MessageBox.Show("Accept device: " + Device.Id, "Accept", MessageBoxButtons.YesNo,
                MessageBoxIcon.Question) == DialogResult.Yes);
        }

        void wclSoftAP_OnDeviceConnectError(object Sender, wclWiFiDirectDevice Device, int Error)
        {
            lbLog.Items.Add("Device " + Device.Name + " connect error: " + Error.ToString("X8"));
        }

        void wclSoftAP_OnDeviceDisconnected(object Sender, wclWiFiDirectDevice Device)
        {
            lbLog.Items.Add("Device disconnected: " + Device.Name);
            lbLog.Items.Add("  Local IP: " + Device.LocalAddress);
            lbLog.Items.Add("  Remote IP: " + Device.RemoteAddress);
        }

        void wclSoftAP_OnDeviceConnected(object Sender, wclWiFiDirectDevice Device)
        {
            lbLog.Items.Add("Device connected: " + Device.Name);
            lbLog.Items.Add("  Local IP: " + Device.LocalAddress);
            lbLog.Items.Add("  Remote IP: " + Device.RemoteAddress);
        }

        void wclSoftAP_OnStopped(object sender, EventArgs e)
        {
            lbLog.Items.Add("Stopped");
        }

        void wclSoftAP_OnStarted(object sender, EventArgs e)
        {
            lbLog.Items.Add("Started");

            String Str;
            Int32 Res = wclSoftAP.GetSsid(out Str);
            if (Res != wclCommon.wclErrors.WCL_E_SUCCESS)
                Str = "Error 0x" + Res.ToString("X8");
            lbLog.Items.Add("SSID: " + Str);

            Res = wclSoftAP.GetPassphrase(out Str);
            if (Res != wclCommon.wclErrors.WCL_E_SUCCESS)
                Str = "Error 0x" + Res.ToString("X8");
            lbLog.Items.Add("Passphrase: " + Str);
        }

        private void btStart_Click(object sender, EventArgs e)
        {
            Int32 Res = wclSoftAP.Start(edSsid.Text, edPassphrase.Text);
            lbLog.Items.Add("Started 0x" + Res.ToString("X8"));
        }

        private void btStop_Click(object sender, EventArgs e)
        {
            Int32 Res = wclSoftAP.Stop();
            lbLog.Items.Add("Stop 0x" + Res.ToString("X8"));
        }

        private void fmMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            wclSoftAP.Stop();
        }

        private void btGetIpSettings_Click(object sender, EventArgs e)
        {
            String Address;
            String Mask;
            String Gateway;
            String Dns1;
            String Dns2;
            Int32 Res = wclSoftAP.GetIpSettings(out Address, out Mask, out Gateway, out Dns1, out Dns2);
            if (Res != wclErrors.WCL_E_SUCCESS)
                lbLog.Items.Add("Read IP settings failed: 0x" + Res.ToString("X8"));
            else
            {
                lbLog.Items.Add("IP settings:");
                lbLog.Items.Add("  Address: " + Address);
                lbLog.Items.Add("  Mask: " + Mask);
                lbLog.Items.Add("  Gateway: " + Gateway);
                lbLog.Items.Add("  Dns1: " + Dns1);
                lbLog.Items.Add("  Dns2: " + Dns2);
            }
        }

        private void btSetIpSettings_Click(object sender, EventArgs e)
        {
            Int32 Res = wclSoftAP.SetIpSettings("192.168.2.10", "255.255.255.0",
                "192.168.2.1", "", "");
            if (Res != wclErrors.WCL_E_SUCCESS)
                lbLog.Items.Add("Write IP settings failed: 0x" + Res.ToString("X8"));
        }

        private void btResetIpSettings_Click(object sender, EventArgs e)
        {
            Int32 Res = wclSoftAP.ResetIpSettings();
            if (Res != wclErrors.WCL_E_SUCCESS)
                lbLog.Items.Add("Reset IP settings failed: 0x" + Res.ToString("X8"));
        }
    }
}