using System;
using System.Windows.Forms;

using wclWiFi;
using wclCommon;

namespace WiFiHostednetwork
{
    public partial class fmMain : Form
    {
        private wclWiFiHostedNetwork Network;
        
        public fmMain()
        {
            InitializeComponent();
        }

        private void fmMain_Load(object sender, EventArgs e)
        {
            Network = new wclWiFiHostedNetwork();
            Network.AfterOpen += new EventHandler(Network_AfterOpen);
            Network.BeforeClose += new EventHandler(Network_BeforeClose);
        }

        void Network_BeforeClose(object sender, EventArgs e)
        {
            MessageBox.Show("WiFi Hosted Network closing");
        }

        void Network_AfterOpen(object sender, EventArgs e)
        {
            MessageBox.Show("WiFi Hosted Network opened");
        }

        private void btHnOpen_Click(object sender, EventArgs e)
        {
            ShowResult(Network.Open());
        }

        private Boolean ShowResult(Int32 Res)
        {
            if (Res != wclErrors.WCL_E_SUCCESS)
            {
                MessageBox.Show("Error: 0x" + Res.ToString("X8"));
                return false;
            }

            return true;
        }

        private void btHnClose_Click(object sender, EventArgs e)
        {
            ShowResult(Network.Close());
        }

        private void btHnGetConnectionSettings_Click(object sender, EventArgs e)
        {
            wclWiFiHostedNetworkConnectionSettings Settings;

            HnClear();

            if (ShowResult(Network.GetConnectionSettings(out Settings)))
            {
                HnAddData("SSID", Settings.Ssid);
                HnAddData("Max. number of peers", Settings.MaxNumberOfPeers.ToString());
            }
        }

        private void HnClear()
        {
            lvHnData.Items.Clear();
        }

        private void btHnClear_Click(object sender, EventArgs e)
        {
            HnClear();
        }

        private void HnAddData(String ParamName, String ParamValue)
        {
            ListViewItem Item = lvHnData.Items.Add(ParamName);
            Item.SubItems.Add(ParamValue);
        }

        private void btHnGetKey_Click(object sender, EventArgs e)
        {
            HnClear();
            
            Byte[] KeyData;
            Boolean IsPassPhrase;
            Boolean Persistent;
            if (ShowResult(Network.GetKey(out KeyData, out IsPassPhrase, out Persistent)))
            {
                String Str;
                if (IsPassPhrase)
                    Str = System.Text.Encoding.ASCII.GetString(KeyData);
                else
                {
                    Str = "";
                    if (KeyData != null)
                        for (Int32 i = 0; i < KeyData.Length; i++)
                            Str = Str + KeyData[i].ToString("X2");
                }

                if (KeyData != null)
                    HnAddData("Key length", KeyData.Length.ToString());
                else
                    HnAddData("Key length", "0");
                HnAddData("Key data", Str);
                HnAddData("Is pass phrase", IsPassPhrase.ToString());
                HnAddData("Persistent", Persistent.ToString());
            }
        }

        private void btHnGetProfile_Click(object sender, EventArgs e)
        {
            HnClear();

            String Xml;
            if (ShowResult(Network.GetProfile(out Xml)))
                HnAddData("Profile XML", Xml);
        }

        private void btHnGetSecuritySettings_Click(object sender, EventArgs e)
        {
            HnClear();

            wclWiFiAuthCipherPair Settings;
            if (ShowResult(Network.GetSecuritySettings(out Settings)))
            {
                HnAddData("Auth algorithm", Settings.AuthAlgorithm.ToString());
                HnAddData("Cipher algorithm", Settings.CipherAlgorithm.ToString());
            }
        }

        private void btHnGetStatus_Click(object sender, EventArgs e)
        {
            HnClear();

            wclWiFiHostedNetworkStatus Status;
            if (ShowResult(Network.GetStatus(out Status)))
            {
                HnAddData("State", Status.State.ToString());
                HnAddData("Interface ID", Status.Id.ToString());
                HnAddData("BSS ID", Status.BssId);
                HnAddData("Phy", Status.Phy.ToString());
                HnAddData("ChannelFrequency", Status.ChannelFrequency.ToString());

                String Address;
                Int32 Res = Network.GetLocalIp(out Address);
                if (Res == wclErrors.WCL_E_SUCCESS)
                    HnAddData("Local IP", Address);
                else
                    HnAddData("Local IP", "Get failed: 0x" + Res.ToString("X8"));

                if (Status.Peers != null)
                {
                    for (Int32 i = 0; i < Status.Peers.Length; i++)
                    {
                        HnAddData("Peer[" + i.ToString() + "] MAC", Status.Peers[i].Mac);
                        Res = Network.GetRemoteIp(Status.Peers[i].Mac, out Address);
                        if (Res == wclErrors.WCL_E_SUCCESS)
                            HnAddData("Peer[" + i.ToString() + "] IP", Address);
                        else
                            HnAddData("Peer[" + i.ToString() + "] IP", "Get failed: 0x" + Res.ToString("X8"));
                        HnAddData("Peer[" + i.ToString() + "] auth state", Status.Peers[i].AuthState.ToString());
                    }
                }
            }
        }

        private void btHnRefreshSecuritySettings_Click(object sender, EventArgs e)
        {
            ShowResult(Network.RefreshSecuritySettings());
        }

        private void btHnSetSettings_Click(object sender, EventArgs e)
        {
            wclWiFiHostedNetworkConnectionSettings Settings = new wclWiFiHostedNetworkConnectionSettings();
            Settings.Ssid = edSsHnid.Text;
            Settings.MaxNumberOfPeers = Convert.ToUInt16(edHnNumberOfPeers.Text);
            ShowResult(Network.SetConnectionSettings(Settings));
        }

        private void btHnSetKey_Click(object sender, EventArgs e)
        {
            Byte[] Tmp = System.Text.Encoding.ASCII.GetBytes(edHnKey.Text);
            Byte[] KeyData = new Byte[Tmp.Length + 1];
            Tmp.CopyTo(KeyData, 0);
            ShowResult(Network.SetKey(KeyData, true, cbHnPersistent.Checked));
        }

        private void btHnEnable_Click(object sender, EventArgs e)
        {
            ShowResult(Network.SetState(true));
        }

        private void btHnDisable_Click(object sender, EventArgs e)
        {
            ShowResult(Network.SetState(false));
        }

        private void fmMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            Network.Close();
        }

        private void btHnGetState_Click(object sender, EventArgs e)
        {
            HnClear();
            
            Boolean State;
            if (ShowResult(Network.GetState(out State)))
                HnAddData("Enabled", State.ToString());
        }

        private void btHnRestart_Click(object sender, EventArgs e)
        {
            ShowResult(Network.Restart());
        }
    }
}