﻿Public Class fmMain

    Private WithEvents wclMobileHotspot As wclMobileHotspot

    Private Sub Trace(Msg As String)
        MessageBox.Show(Msg)
    End Sub

    Private Sub Trace(Msg As String, [Error] As Int32)
        Trace(Msg + ". Error: 0x" + [Error].ToString("X8"))
    End Sub

    Private Sub fmMain_Load(sender As System.Object, e As System.EventArgs) Handles MyBase.Load
        wclMobileHotspot = New wclMobileHotspot()
        cbBand.SelectedIndex = 0
    End Sub

    Private Sub wclMobileHotspot_OnClosed(sender As Object, e As System.EventArgs) Handles wclMobileHotspot.OnClosed
        Trace("Mobile hotspot closed.")

        edSsid.Text = ""
        edPassword.Text = ""
    End Sub

    Private Sub wclMobileHotspot_OnOpened(sender As Object, e As System.EventArgs) Handles wclMobileHotspot.OnOpened
        Trace("Mobile hotspot opened.")
        Dim Ssid As String = ""
        If wclMobileHotspot.GetSsid(Ssid) = wclErrors.WCL_E_SUCCESS Then
            edSsid.Text = Ssid
        End If
        Dim Passphrase As String = ""
        If wclMobileHotspot.GetPassphrase(Passphrase) = wclErrors.WCL_E_SUCCESS Then
            edPassword.Text = Passphrase
        End If
    End Sub

    Private Sub fmMain_FormClosed(sender As System.Object, e As System.Windows.Forms.FormClosedEventArgs) Handles MyBase.FormClosed
        wclMobileHotspot.Close()
        wclMobileHotspot = Nothing
    End Sub

    Private Sub btOpen_Click(sender As System.Object, e As System.EventArgs) Handles btOpen.Click
        Dim Res As Int32 = wclMobileHotspot.Open()
        If Res <> wclErrors.WCL_E_SUCCESS Then
            Trace("Open failed", Res)
        End If
    End Sub

    Private Sub btClose_Click(sender As System.Object, e As System.EventArgs) Handles btClose.Click
        Dim Res As Int32 = wclMobileHotspot.Close()
        If Res <> wclErrors.WCL_E_SUCCESS Then
            Trace("Close failed", Res)
        End If
    End Sub

    Private Sub btStart_Click(sender As System.Object, e As System.EventArgs) Handles btStart.Click
        Dim Res As Int32 = wclMobileHotspot.Start()
        If Res <> wclErrors.WCL_E_SUCCESS Then
            Trace("Start failed", Res)
        End If
    End Sub

    Private Sub btStop_Click(sender As System.Object, e As System.EventArgs) Handles btStop.Click
        Dim Res As Int32 = wclMobileHotspot.Stop()
        If Res <> wclErrors.WCL_E_SUCCESS Then
            Trace("Stop failed", Res)
        End If
    End Sub

    Private Sub btGetState_Click(sender As System.Object, e As System.EventArgs) Handles btGetState.Click
        Dim State As wclMobileHotspotState
        Dim Res As Int32 = wclMobileHotspot.GetState(State)
        If Res <> wclErrors.WCL_E_SUCCESS Then
            Trace("Get state failed", Res)
        Else
            Select State
                Case wclMobileHotspotState.mhsUnknown
                    Trace("State: unknown")
                Case wclMobileHotspotState.mhsOn
                    Trace("State: on")
                Case wclMobileHotspotState.mhsOff
                    Trace("State: off")
                Case wclMobileHotspotState.mhsInTransition
                    Trace("State: switching")
            End Select
        End If
    End Sub

    Private Sub btGetSsid_Click(sender As System.Object, e As System.EventArgs) Handles btGetSsid.Click
        Dim Ssid As String = ""
        Dim Res As Int32 = wclMobileHotspot.GetSsid(Ssid)
        If Res <> wclErrors.WCL_E_SUCCESS Then
            Trace("Get SSID failed", Res)
        Else
            edSsid.Text = Ssid
        End If
    End Sub

    Private Sub btSetSsid_Click(sender As System.Object, e As System.EventArgs) Handles btSetSsid.Click
        Dim Res As Int32 = wclMobileHotspot.SetSsid(edSsid.Text)
        If Res <> wclErrors.WCL_E_SUCCESS Then
            Trace("Set SSID failed", Res)
        End If
    End Sub

    Private Sub btGetPwd_Click(sender As System.Object, e As System.EventArgs) Handles btGetPwd.Click
        Dim Pwd As String = ""
        Dim Res As Int32 = wclMobileHotspot.GetPassphrase(Pwd)
        If Res <> wclErrors.WCL_E_SUCCESS Then
            Trace("Get passphrase failed", Res)
        Else
            edPassword.Text = Pwd
        End If
    End Sub

    Private Sub btSetPwd_Click(sender As System.Object, e As System.EventArgs) Handles btSetPwd.Click
        Dim Res As Int32 = wclMobileHotspot.SetPassphrase(edPassword.Text)
        If Res <> wclErrors.WCL_E_SUCCESS Then
            Trace("Set passphrase failed", Res)
        End If
    End Sub

    Private Sub btMaxClients_Click(sender As System.Object, e As System.EventArgs) Handles btMaxClients.Click
        Dim Count As UInt32
        Dim Res As Int32 = wclMobileHotspot.GetMaxClientCount(Count)
        If Res <> wclErrors.WCL_E_SUCCESS Then
            Trace("Get max client connections failed", Res)
        Else
            Trace("Maximum allowed connections: " + Count.ToString())
        End If
    End Sub

    Private Sub btConnected_Click(sender As System.Object, e As System.EventArgs) Handles btConnected.Click
        Dim Count As UInt32
        Dim Res As Int32 = wclMobileHotspot.GetClientCount(Count)
        If Res <> wclErrors.WCL_E_SUCCESS Then
            Trace("Get connectioned clients count failed", Res)
        Else
            Trace("Connected clients: " + Count.ToString())
        End If
    End Sub

    Private Sub btIsSupported_Click(sender As System.Object, e As System.EventArgs) Handles btIsSupported.Click
        Dim Band As wclMobileHotspotBand
        Select cbBand.SelectedIndex
            Case 0
                    Band = wclMobileHotspotBand.mhbAuto
            Case 1
                    Band = wclMobileHotspotBand.mhbTwoPointFourGigahertz
            Case Else
                Band = wclMobileHotspotBand.mhbFiveGigahertz
        End Select

        Dim Supp As Boolean
        Dim Res As Int32 = wclMobileHotspot.IsBandSupported(Band, Supp)
        If Res <> wclErrors.WCL_E_SUCCESS Then
            Trace("Check band failed", Res)
        Else
            If Supp Then
                MessageBox.Show("Band is supported")
            Else
                MessageBox.Show("Band is NOT supported")
            End If
        End If
    End Sub

    Private Sub btConnectedClients_Click(sender As System.Object, e As System.EventArgs) Handles btConnectedClients.Click
        lvConnectedClients.Items.Clear()
        Dim Clients() As wclMobileHotspotClient = Nothing
        Dim Res As Int32 = wclMobileHotspot.GetClients(Clients)
        If Res <> wclErrors.WCL_E_SUCCESS Then
            Trace("Get connected clients failed", Res)
        Else
            If Clients IsNot Nothing AndAlso Clients.Length > 0 Then
                For Each Client As wclMobileHotspotClient In Clients
                    Dim Item As ListViewItem = lvConnectedClients.Items.Add(Client.Mac)
                    Item.SubItems.Add(Client.Name)
                Next Client
            End If
        End If
    End Sub

    Private Sub btCheckTimeout_Click(sender As System.Object, e As System.EventArgs) Handles btCheckTimeout.Click
        Dim Enabled As Boolean = False
        Dim Res As Int32 = wclMobileHotspot.IsNoConnectionsTimeoutEnabled(Enabled)
        If Res <> wclErrors.WCL_E_SUCCESS Then
            Trace("Get no connection timeout status failed", Res)
        Else
            If Enabled Then
                MessageBox.Show("No connection timeout enabled")
            Else
                MessageBox.Show("No connection timeout disabled")
            End If
        End If
    End Sub

    Private Sub btEnableTimeout_Click(sender As System.Object, e As System.EventArgs) Handles btEnableTimeout.Click
        Dim Res As Int32 = wclMobileHotspot.EnableNoConnectionsTimeout()
        If Res <> wclErrors.WCL_E_SUCCESS Then
            Trace("Enable No Connection Timeout failed", Res)
        End If
    End Sub

    Private Sub btDisableTimeout_Click(sender As System.Object, e As System.EventArgs) Handles btDisableTimeout.Click
        Dim Res As Int32 = wclMobileHotspot.DisableNoConnectionsTimeout()
        If Res <> wclErrors.WCL_E_SUCCESS Then
            Trace("Disable No Connection Timeout failed", Res)
        End If
    End Sub
End Class
