Public Class fmMain
    Private WithEvents Network As wclWiFi.wclWiFiHostedNetwork

    Private Sub fmMain_FormClosed(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosedEventArgs) Handles Me.FormClosed
        Network.Close()
    End Sub

    Private Sub fmMain_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        Network = New wclWiFi.wclWiFiHostedNetwork()
    End Sub

    Private Sub Network_AfterOpen(ByVal sender As Object, ByVal e As System.EventArgs) Handles Network.AfterOpen
        MessageBox.Show("WiFi Hosted Network opened")
    End Sub

    Private Sub Network_BeforeClose(ByVal sender As Object, ByVal e As System.EventArgs) Handles Network.BeforeClose
        MessageBox.Show("WiFi Hosted Network closing")
    End Sub

    Private Sub btHnOpen_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btHnOpen.Click
        ShowResult(Network.Open())
    End Sub

    Private Function ShowResult(ByVal Res As Int32) As Boolean
        If Res <> wclCommon.wclErrors.WCL_E_SUCCESS Then
            MessageBox.Show("Error: 0x" + Res.ToString("X8"))
            Return False
        End If

        Return True
    End Function

    Private Sub btHnClose_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btHnClose.Click
        ShowResult(Network.Close())
    End Sub

    Private Sub btHnGetConnectionSettings_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btHnGetConnectionSettings.Click
        Dim Settings As wclWiFi.wclWiFiHostedNetworkConnectionSettings = Nothing

        HnClear()

        If ShowResult(Network.GetConnectionSettings(Settings)) Then
            HnAddData("SSID", Settings.Ssid)
            HnAddData("Max. number of peers", Settings.MaxNumberOfPeers.ToString())
        End If
    End Sub

    Private Sub HnClear()
        lvHnData.Items.Clear()
    End Sub

    Private Sub btHnClear_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btHnClear.Click
        HnClear()
    End Sub

    Private Sub HnAddData(ByVal ParamName As String, ByVal ParamValue As String)
        Dim Item As ListViewItem = lvHnData.Items.Add(ParamName)
        Item.SubItems.Add(ParamValue)
    End Sub

    Private Sub btHnGetKey_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btHnGetKey.Click
        HnClear()

        Dim KeyData() As Byte = Nothing
        Dim IsPassPhrase As Boolean = False
        Dim Persistent As Boolean = False
        If ShowResult(Network.GetKey(KeyData, IsPassPhrase, Persistent)) Then
            Dim Str As String
            If IsPassPhrase Then
                Str = System.Text.Encoding.ASCII.GetString(KeyData)
            Else
                Str = ""
                If KeyData IsNot Nothing Then
                    Dim i As Int32
                    For i = 0 To KeyData.Length - 1
                        Str = Str + KeyData(i).ToString("X2")
                    Next i
                End If
            End If

            If KeyData IsNot Nothing Then
                HnAddData("Key length", KeyData.Length.ToString())
            Else
                HnAddData("Key length", "0")
            End If
            HnAddData("Key data", Str)
            HnAddData("Is pass phrase", IsPassPhrase.ToString())
            HnAddData("Persistent", Persistent.ToString())
        End If
    End Sub

    Private Sub btHnGetProfile_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btHnGetProfile.Click
        HnClear()

        Dim Xml As String = ""
        If ShowResult(Network.GetProfile(Xml)) Then HnAddData("Profile XML", Xml)
    End Sub

    Private Sub btHnGetSecuritySettings_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btHnGetSecuritySettings.Click
        HnClear()

        Dim Settings As wclWiFi.wclWiFiAuthCipherPair
        If ShowResult(Network.GetSecuritySettings(Settings)) Then
            HnAddData("Auth algorithm", Settings.AuthAlgorithm.ToString())
            HnAddData("Cipher algorithm", Settings.CipherAlgorithm.ToString())
        End If
    End Sub

    Private Sub btHnGetStatus_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btHnGetStatus.Click
        HnClear()

        Dim Status As wclWiFi.wclWiFiHostedNetworkStatus = Nothing
        If ShowResult(Network.GetStatus(Status)) Then
            HnAddData("State", Status.State.ToString())
            HnAddData("Interface ID", Status.Id.ToString())
            HnAddData("BSS ID", Status.BssId)
            HnAddData("Phy", Status.Phy.ToString())
            HnAddData("ChannelFrequency", Status.ChannelFrequency.ToString())

            Dim Address As String = ""
            Dim Res As Integer = Network.GetLocalIp(Address)
            If Res = wclCommon.wclErrors.WCL_E_SUCCESS Then
                HnAddData("Local IP", Address)
            Else
                HnAddData("Local IP", "Get failed: 0x" + Res.ToString("X8"))
            End If

            If Status.Peers IsNot Nothing Then
                Dim i As Int32
                For i = 0 To Status.Peers.Length - 1
                    HnAddData("Peer[" + i.ToString() + "] MAC", Status.Peers(i).Mac)
                    Res = Network.GetRemoteIp(Status.Peers(i).Mac, Address)
                    If Res = wclCommon.wclErrors.WCL_E_SUCCESS Then
                        HnAddData("Peer[" + i.ToString() + "] IP", Address)
                    Else
                        HnAddData("Peer[" + i.ToString() + "] IP", "Get failed: 0x" + Res.ToString("X8"))
                    End If
                    HnAddData("Peer[" + i.ToString() + "] auth state", Status.Peers(i).AuthState.ToString())
                Next i
            End If
        End If
    End Sub

    Private Sub btHnRefreshSecuritySettings_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btHnRefreshSecuritySettings.Click
        ShowResult(Network.RefreshSecuritySettings())
    End Sub

    Private Sub btHnSetSettings_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btHnSetSettings.Click
        Dim Settings As wclWiFi.wclWiFiHostedNetworkConnectionSettings = New wclWiFi.wclWiFiHostedNetworkConnectionSettings()
        Settings.Ssid = edHnSsid.Text
        Settings.MaxNumberOfPeers = Convert.ToUInt16(edHnNumberOfPeers.Text)
        ShowResult(Network.SetConnectionSettings(Settings))
    End Sub

    Private Sub btHnSetKey_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btHnSetKey.Click
        Dim Tmp() As Byte = System.Text.Encoding.ASCII.GetBytes(edHnKey.Text)
        Dim KeyData = New Byte(Tmp.Length) {}
        Tmp.CopyTo(KeyData, 0)
        ShowResult(Network.SetKey(KeyData, True, cbHnPersistent.Checked))
    End Sub

    Private Sub btHnEnable_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btHnEnable.Click
        ShowResult(Network.SetState(True))
    End Sub

    Private Sub btHnDisable_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btHnDisable.Click
        ShowResult(Network.SetState(False))
    End Sub

    Private Sub btHnGetState_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btHnGetState.Click
        HnClear()

        Dim State As Boolean
        If ShowResult(Network.GetState(State)) Then HnAddData("Enabled", State.ToString())
    End Sub

    Private Sub btHnRestart_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btHnRestart.Click
        ShowResult(Network.Restart())
    End Sub
End Class
