Public Class fmMain
    Private WithEvents Client As wclWiFi.wclWiFiClient
    Private ProfilesManager As wclWiFi.wclWiFiProfilesManager

    Private Sub fmMain_FormClosed(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosedEventArgs) Handles Me.FormClosed
        Client.Close()
    End Sub

    Private Sub fmMain_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        cbClientBssType.SelectedIndex = 2
        cbClientConnect.SelectedIndex = 0

        Client = New wclWiFi.wclWiFiClient
        ProfilesManager = New wclWiFi.wclWiFiProfilesManager
    End Sub

    Private Sub Client_AfterOpen(ByVal sender As Object, ByVal e As System.EventArgs) Handles Client.AfterOpen
        MessageBox.Show("WiFi client opened")

        ProfilesManager.Open()
    End Sub

    Private Sub Client_BeforeClose(ByVal sender As Object, ByVal e As System.EventArgs) Handles Client.BeforeClose
        ProfilesManager.Close()

        ClientClearInterfaces()
        ClientClearNetworks()
        ClientClearBss()
        ClientClearProfiles()

        MessageBox.Show("WiFi client is closing")
    End Sub

    Private Function ShowResult(ByVal Res As Int32) As Boolean
        If Res = wclCommon.wclErrors.WCL_E_SUCCESS Then
            Return True
        End If

        MessageBox.Show("Error: 0x" + Res.ToString("X8"))
        Return False
    End Function

    Private Sub btClientOpen_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btClientOpen.Click
        ShowResult(Client.Open())
    End Sub

    Private Sub btClientClose_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btClientClose.Click
        ShowResult(Client.Close())
    End Sub

    Private Sub ClientClearInterfaces()
        lvIClientnterfaces.Items.Clear()
    End Sub

    Private Sub btClientEnumInterfaces_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btClientEnumInterfaces.Click
        Dim Interfaces() As wclWiFi.wclWiFiInterfaceData = Nothing
        ClientClearInterfaces()

        If ShowResult(Client.EnumInterfaces(Interfaces)) Then
            If Interfaces IsNot Nothing Then
                Dim i As Int32
                For i = 0 To Interfaces.Length - 1
                    Dim Item As ListViewItem = lvIClientnterfaces.Items.Add(Interfaces(i).Id.ToString())
                    Item.SubItems.Add(Interfaces(i).Description)
                    Item.SubItems.Add(Interfaces(i).Primary.ToString())
                Next i
            End If
        End If
    End Sub

    Private Function ClientGetInterfaceId(ByRef Id As Guid) As Boolean
        Id = Guid.Empty

        If lvIClientnterfaces.SelectedItems.Count = 0 Then
            MessageBox.Show("Select interface")
            Return False
        End If

        Id = New Guid(lvIClientnterfaces.SelectedItems(0).Text)
        Return True
    End Function

    Private Sub ClientClearNetworks()
        lvClientNetworks.Items.Clear()
    End Sub

    Private Sub ClientClearBss()
        lvClientBss.Items.Clear()
    End Sub

    Private Sub ClientClearProfiles()
        lvClientProfiles.Items.Clear()
    End Sub

    Private Sub btClientEnumNetworks_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btClientEnumNetworks.Click
        Dim Id As Guid = Guid.Empty

        ClientClearNetworks()
        If ClientGetInterfaceId(Id) Then
            Dim Filters As wclWiFi.wclWiFiAvailableNetworkFilter = 0
            If cbClientAdHocProfiles.Checked Then
                Filters = Filters Or wclWiFi.wclWiFiAvailableNetworkFilter.ffIncludeAllAdhocProfiles
            End If
            If cbClientHiddenProfiles.Checked Then
                Filters = Filters Or wclWiFi.wclWiFiAvailableNetworkFilter.ffIncludeAllManualHiddenProfiles
            End If

            Dim Networks() As wclWiFi.wclWiFiAvailableNetwork = Nothing
            If ShowResult(Client.EnumAvailableNetworks(Id, Filters, Networks)) Then
                If Networks IsNot Nothing Then
                    Dim i As Int32

                    For i = 0 To Networks.Length - 1
                        Dim Item As ListViewItem = lvClientNetworks.Items.Add(Networks(i).ProfileName)
                        Item.SubItems.Add(Networks(i).Ssid)
                        Item.SubItems.Add(Networks(i).BssType.ToString())
                        Item.SubItems.Add(Networks(i).NumberOfBssids.ToString())
                        Item.SubItems.Add(Networks(i).NetworkConnectable.ToString())
                        Item.SubItems.Add(Networks(i).NotConnectableReason.ToString())
                        Item.SubItems.Add(Networks(i).PhyTypes.ToString())
                        Item.SubItems.Add(Networks(i).MorePhyTypes.ToString())
                        Item.SubItems.Add(Networks(i).SignalQuality.ToString())
                        Item.SubItems.Add(Networks(i).SecurityEnabled.ToString())
                        Item.SubItems.Add(Networks(i).DefaultAuthAlgorithm.ToString())
                        Item.SubItems.Add(Networks(i).DefaultCipherAlgorithm.ToString())
                        Item.SubItems.Add(Networks(i).Flags.ToString())
                    Next i
                End If
            End If
        End If
    End Sub

    Private Sub btClientScan_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btClientScan.Click
        Dim Id As Guid = Guid.Empty
        If ClientGetInterfaceId(Id) Then
            If ShowResult(Client.Scan(Id, edClientScanSsid.Text)) Then
                MessageBox.Show("Scan started. Use EventsDemo to see scan events.")
            End If
        End If
    End Sub

    Private Sub btClientEnumBss_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btClientEnumBss.Click
        ClientClearBss()
        Dim Id As Guid = Guid.Empty

        If ClientGetInterfaceId(Id) Then
            Dim Ssid As String = ""

            If cbClientUseSsid.Checked Then
                If lvClientNetworks.SelectedItems.Count = 0 Then
                    MessageBox.Show("Select network")
                    Return
                End If

                Ssid = lvClientNetworks.SelectedItems(0).SubItems(1).Text
            End If

            Dim BssType As wclWiFi.wclWiFiBssType = cbClientBssType.SelectedIndex ' wclWiFi.wclWiFiBssType
            Dim BssList() As wclWiFi.wclWiFiBss = Nothing

            If ShowResult(Client.EnumBss(Id, Ssid, BssType, cbClientSecurityEnabled.Checked, BssList)) Then
                If BssList IsNot Nothing Then
                    Dim i As Int32
                    For i = 0 To BssList.Length - 1
                        Dim Item As ListViewItem = lvClientBss.Items.Add(BssList(i).Ssid)
                        Item.SubItems.Add(BssList(i).PhyId.ToString())
                        Item.SubItems.Add(BssList(i).Mac)
                        Item.SubItems.Add(BssList(i).BssType.ToString())
                        Item.SubItems.Add(BssList(i).PhyType.ToString())
                        Item.SubItems.Add(BssList(i).Rssi.ToString())
                        Item.SubItems.Add(BssList(i).LinkQuality.ToString())
                        Item.SubItems.Add(BssList(i).InRegDomain.ToString())
                        Item.SubItems.Add(BssList(i).BeaconPeriod.ToString())
                        Item.SubItems.Add(BssList(i).Timestamp.ToString())
                        Item.SubItems.Add(BssList(i).HostTimestamp.ToString())
                        Item.SubItems.Add(BssList(i).Capability.ToString())
                        Item.SubItems.Add(BssList(i).ChCenterFrequency.ToString())

                        Dim s As String = ""
                        If BssList(i).IeRaw IsNot Nothing AndAlso BssList(i).IeRaw.Length > 0 Then
                            Dim X As Int32
                            For X = 0 To BssList(i).IeRaw.Length - 1
                                s = s + BssList(i).IeRaw(X).ToString("X2")
                            Next X
                        End If
                        Item.SubItems.Add(s)
                    Next i
                End If
            End If
        End If
    End Sub

    Private Sub btClientDisconnect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btClientDisconnect.Click
        Dim Id As Guid = Guid.Empty
        If ClientGetInterfaceId(Id) Then
            ShowResult(Client.Disconnect(Id))
        End If
    End Sub

    Private Sub btClientGetShowDeniedNetworks_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btClientGetShowDeniedNetworks.Click
        Dim Enabled As Boolean = False
        If ShowResult(Client.AcmGetShowDeniedNetworks(Enabled)) Then
            If Enabled Then
                MessageBox.Show("Show denied networks")
            Else
                MessageBox.Show("Do not show denied networks")
            End If
        End If
    End Sub

    Private Sub btClientSetShowDeniedNetworks_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btClientSetShowDeniedNetworks.Click
        Dim Res As DialogResult = MessageBox.Show("Show denied networks?", "Show denied networks", MessageBoxButtons.YesNoCancel, _
            MessageBoxIcon.Question)
        If Res = DialogResult.Cancel Then
            Return
        End If
        ShowResult(Client.AcmSetShowDeniedNetworks(Res = DialogResult.Yes))
    End Sub

    Private Sub btClientGetPowerSettings_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btClientGetPowerSettings.Click
        Dim Setting As wclWiFi.wclWiFiPowerSetting
        If ShowResult(Client.AcmGetPowerSetting(Setting)) Then
            MessageBox.Show(Setting.ToString())
        End If
    End Sub

    Private Sub btClientGetOnlyUseGpProfiles_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btClientGetOnlyUseGpProfiles.Click
        Dim Enabled As Boolean
        If ShowResult(Client.AcmGetOnlyUseGpProfiles(Enabled)) Then
            If Enabled Then
                MessageBox.Show("Only use GP profiles ENABLED")
            Else
                MessageBox.Show("Only use GP profiles DISABLED")
            End If
        End If
    End Sub

    Private Sub btClientGetAllowExplicitCreds_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btClientGetAllowExplicitCreds.Click
        Dim Enabled As Boolean
        If ShowResult(Client.AcmGetAllowExplicitCreds(Enabled)) Then
            If Enabled Then
                MessageBox.Show("Explicit creds ALLOWED")
            Else
                MessageBox.Show("Explicit creds NOT ALLOWED")
            End If
        End If
    End Sub

    Private Sub btClientSetAllowExplicitCreds_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btClientSetAllowExplicitCreds.Click
        Dim Res As DialogResult = MessageBox.Show("Allow explicit creds?", "Allow explicit creds?", MessageBoxButtons.YesNoCancel, _
            MessageBoxIcon.Question)
        If Res = DialogResult.Cancel Then
            Return
        End If
        ShowResult(Client.AcmSetAllowExplicitCreds(Res = DialogResult.Yes))
    End Sub

    Private Sub btClientGetBlockPeriod_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btClientGetBlockPeriod.Click
        Dim Period As UInt32
        If ShowResult(Client.AcmGetBlockPeriod(Period)) Then
            MessageBox.Show("Block period: " + Period.ToString())
        End If
    End Sub

    Private Sub btClientSetBlockPeriod_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btClientSetBlockPeriod.Click
        ' SET IT TO 10 here
        Dim Period As UInt32 = 10
        ShowResult(Client.AcmSetBlockPeriod(Period))
    End Sub

    Private Sub btClientGetVirtualStationExtensibility_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btClientGetVirtualStationExtensibility.Click
        Dim Allowed As Boolean
        If ShowResult(Client.AcmGetVirtualStationExtensibility(Allowed)) Then
            If Allowed Then
                MessageBox.Show("Virtual station extensibility ALLOWED")
            Else
                MessageBox.Show("Virtual station extensibility NOT ALLOWED")
            End If
        End If
    End Sub

    Private Sub btClientSetVirtualStationExtensibility_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btClientSetVirtualStationExtensibility.Click
        Dim Res As DialogResult = MessageBox.Show("Allow wirtual station extensibility?", "Allow wirtual station extensibility?", _
            MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question)
        If Res = DialogResult.Cancel Then
            Return
        End If

        ShowResult(Client.AcmSetVirtualStationExtensibility(Res = DialogResult.Yes))
    End Sub

    Private Sub btClientGetInterfaceState_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btClientGetInterfaceState.Click
        Dim Id As Guid
        If ClientGetInterfaceId(Id) Then
            Dim Iface As wclWiFi.wclWiFiInterface = New wclWiFi.wclWiFiInterface(Id)
            If ShowResult(Iface.Open()) Then
                Try
                    Dim State As wclWiFi.wclWiFiInterfaceState
                    If ShowResult(Iface.GetState(State)) Then
                        MessageBox.Show("Interface state: " + State.ToString())
                    End If
                Finally
                    Iface.Close()
                End Try
            End If
        End If
    End Sub

    Private Sub btClientEnumProfiles_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btClientEnumProfiles.Click
        ClientEnumProfiles()
    End Sub

    Private Sub btClientDeleteProfile_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btClientDeleteProfile.Click
        Dim Id As Guid
        If ClientGetInterfaceId(Id) Then
            Dim Profile As String = ""
            If ClientGetProfile(Profile) Then
                If ShowResult(ProfilesManager.DeleteProfile(Id, Profile)) Then
                    ClientEnumProfiles()
                End If
            End If
        End If
    End Sub

    Private Sub ClientEnumProfiles()
        ClientClearProfiles()

        Dim Id As Guid
        If ClientGetInterfaceId(Id) Then
            Dim Profiles() As wclWiFi.wclWiFiProfileData = Nothing
            If ShowResult(ProfilesManager.GetProfileList(Id, Profiles)) Then
                If Profiles IsNot Nothing Then
                    Dim I As Int32
                    For I = 0 To Profiles.Length - 1
                        Dim Item As ListViewItem = lvClientProfiles.Items.Add(Profiles(I).Name)
                        Item.SubItems.Add(Profiles(I).Flags.ToString())
                    Next I
                End If
            End If
        End If
    End Sub

    Private Function ClientGetProfile(ByRef Name As String) As Boolean
        Name = ""

        If lvClientProfiles.SelectedItems.Count = 0 Then
            MessageBox.Show("Select profile:")
            Return False
        End If

        Name = lvClientProfiles.SelectedItems(0).Text

        Return True
    End Function

    Private Sub btClientRenameProfile_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btClientRenameProfile.Click
        Dim Id As Guid
        If ClientGetInterfaceId(Id) Then
            Dim Profile As String = ""
            If ClientGetProfile(Profile) Then
                'Use New_<OLD_PROFILE_NAME>
                If ShowResult(ProfilesManager.RenameProfile(Id, Profile, "NEW_" + Profile)) Then
                    ClientEnumProfiles()
                End If
            End If
        End If
    End Sub

    Private Sub btClientGetProfileXml_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btClientGetProfileXml.Click
        Dim Id As Guid
        If ClientGetInterfaceId(Id) Then
            Dim Profile As String = ""
            If ClientGetProfile(Profile) Then
                Dim Flags As wclWiFi.wclWiFiProfileFlag = 0
                If cbPlainText.Checked Then
                    Flags = Flags Or wclWiFi.wclWiFiProfileFlag.pfGetPlaintextKey
                End If
                Dim Xml As String = ""
                If ShowResult(ProfilesManager.GetProfile(Id, Profile, Flags, Xml)) Then
                    saveFileDialog.FileName = Profile + ".xml"
                    If saveFileDialog.ShowDialog = DialogResult.OK Then
                        System.IO.File.WriteAllText(saveFileDialog.FileName, Xml)
                    End If
                End If
            End If
        End If
    End Sub

    Private Sub btClientSetProfileXml_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btClientSetProfileXml.Click
        Dim Id As Guid
        If ClientGetInterfaceId(Id) Then
            openFileDialog.FileName = ""
            If openFileDialog.ShowDialog = DialogResult.OK Then
                Dim Xml As String
                Xml = System.IO.File.ReadAllText(openFileDialog.FileName)
                ShowResult(ProfilesManager.SetProfile(Id, 0, Xml, True))
            End If
        End If
    End Sub

    Private Sub btClientConnect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btClientConnect.Click
        Select Case cbClientConnect.SelectedIndex
            Case 0
                ClientConnectUsingNetworkProfile()
            Case 1
                ClientConnectUsingSelectedprofile()
            Case 2
                ClientConnectUsingSelectedNetwork()
            Case 3
                ClientConenctUsingTemporaryProfile()
        End Select
    End Sub

    Private Sub ClientConenctUsingTemporaryProfile()
        Dim Id As Guid
        If ClientGetInterfaceId(Id) Then
            If lvClientNetworks.SelectedItems.Count = 0 Then
                MessageBox.Show("Select network")
            Else
                If cbUseBssMac.Checked AndAlso lvClientBss.SelectedItems.Count = 0 Then
                    MessageBox.Show("Select Bss")
                Else
                    Dim Mac As String = ""
                    If cbUseBssMac.Checked Then
                        Mac = lvClientBss.SelectedItems(0).SubItems(2).Text
                    End If

                    Dim Ssid As String = lvClientNetworks.SelectedItems(0).SubItems(1).Text

                    Dim Utf8 As Byte() = System.Text.Encoding.UTF8.GetBytes(Ssid)
                    Dim Hex As String = ""
                    For Each b As Byte In Utf8
                        Hex = Hex + b.ToString("X2")
                    Next

                    Dim AuthAlg As String
                    If lvClientNetworks.SelectedItems(0).SubItems(11).Text = "auOpen" Then
                        AuthAlg = "open"
                    Else
                        If lvClientNetworks.SelectedItems(0).SubItems(11).Text = "auSharedKey" Then
                            AuthAlg = "shared"
                        Else
                            If lvClientNetworks.SelectedItems(0).SubItems(11).Text = "auWpa" Then
                                AuthAlg = "WPA"
                            Else
                                If lvClientNetworks.SelectedItems(0).SubItems(11).Text = "auWpaPsk" Then
                                    AuthAlg = "WPAPSK"
                                Else
                                    If lvClientNetworks.SelectedItems(0).SubItems(11).Text = "auRsna" Then
                                        AuthAlg = "WPA2"
                                    Else
                                        If lvClientNetworks.SelectedItems(0).SubItems(11).Text = "auWpa3" Then
                                            AuthAlg = "WPA3ENT192"
                                        Else
                                            If lvClientNetworks.SelectedItems(0).SubItems(11).Text = "auWpa3Sae" Then
                                                AuthAlg = "WPA3SAE"
                                            Else
                                                If lvClientNetworks.SelectedItems(0).SubItems(11).Text = "auWpa3Ent" Then
                                                    AuthAlg = "WPA3ENT"
                                                Else
                                                    If lvClientNetworks.SelectedItems(0).SubItems(11).Text = "auOwe" Then
                                                        AuthAlg = "OWE"
                                                    Else
                                                        AuthAlg = "WPA2PSK"
                                                    End If
                                                End If
                                            End If
                                        End If
                                    End If
                                End If
                            End If
                        End If
                    End If

                    Dim CipherAlg As String
                    If lvClientNetworks.SelectedItems(0).SubItems(12).Text = "caNone" Then
                        CipherAlg = "none"
                    Else
                        If lvClientNetworks.SelectedItems(0).SubItems(12).Text = "caTkip" Then
                            CipherAlg = "TKIP"
                        Else
                            If lvClientNetworks.SelectedItems(0).SubItems(12).Text = "caWep40" Or _
                               lvClientNetworks.SelectedItems(0).SubItems(12).Text = "caWep104" Or _
                               lvClientNetworks.SelectedItems(0).SubItems(12).Text = "caWep" _
                            Then
                                CipherAlg = "WEP"
                            Else
                                If lvClientNetworks.SelectedItems(0).SubItems(12).Text = "caGcmp256" Then
                                    CipherAlg = "GCMP256"
                                Else
                                    CipherAlg = "AES"
                                End If
                            End If
                        End If
                    End If

                    Dim Xml As String = "<?xml version='1.0'?>" + _
                        "<WLANProfile xmlns='http://www.microsoft.com/networking/WLAN/profile/v1'>" + _
                        "<name>" + Ssid + "</name>" + _
                        "<SSIDConfig>" + _
                        "<SSID>" + _
                        "<hex>" + Hex + "</hex>" + _
                        "<name>" + Ssid + "</name>" + _
                        "</SSID>" + _
                        "</SSIDConfig>" + _
                        "<connectionType>ESS</connectionType>" + _
                        "<connectionMode>manual</connectionMode>" + _
                        "<autoSwitch>false</autoSwitch>" + _
                        "<MSM>" + _
                        "<security>" + _
                        "<authEncryption>" + _
                        "<authentication>" + AuthAlg + "</authentication>" + _
                        "<encryption>" + CipherAlg + "</encryption>" + _
                        "<useOneX>false</useOneX>" + _
                        "</authEncryption>" + _
                        "<sharedKey>" + _
                        "<keyType>passPhrase</keyType>" + _
                        "<protected>false</protected>" + _
                        "<keyMaterial>" + edClientPasskey.Text + "</keyMaterial>" + _
                        "</sharedKey>" + _
                        "</security>" + _
                        "</MSM>" + _
                        "</WLANProfile>"

                    ShowResult(Client.Connect(Id, wclWiFi.wclWiFiConnectionMode.cmTemporaryProfile, Xml, _
                        Ssid, ClientGetNetworkBss(), 0, Mac))
                End If
            End If
        End If
    End Sub

    Private Function ClientGetNetworkBss() As wclWiFi.wclWiFiBssType
        If lvClientNetworks.SelectedItems.Count = 0 Then
            Return cbClientBssType.SelectedIndex
        End If

        If lvClientNetworks.SelectedItems(0).SubItems(2).Text = "bssInfrastructure" Then
            Return wclWiFi.wclWiFiBssType.bssInfrastructure
        End If

        If lvClientNetworks.SelectedItems(0).SubItems(2).Text = "bssIndependent" Then
            Return wclWiFi.wclWiFiBssType.bssIndependent
        End If

        Throw New Exception("Inavlid BSS type")
    End Function

    Private Sub ClientConnectUsingNetworkProfile()
        Dim Id As Guid
        If ClientGetInterfaceId(Id) Then
            If lvClientNetworks.SelectedItems.Count = 0 Then
                MessageBox.Show("Select network")
            Else
                If cbUseBssMac.Checked AndAlso lvClientBss.SelectedItems.Count = 0 Then
                    MessageBox.Show("Select Bss")
                Else
                    Dim Mac As String = ""
                    If cbUseBssMac.Checked Then
                        Mac = lvClientBss.SelectedItems(0).SubItems(2).Text
                    End If

                    ShowResult(Client.Connect(Id, wclWiFi.wclWiFiConnectionMode.cmProfile, _
                        lvClientNetworks.SelectedItems(0).Text, "", ClientGetNetworkBss(), 0, Mac))
                End If
            End If
        End If
    End Sub

    Private Sub ClientConnectUsingSelectedNetwork()
        Dim Id As Guid
        If ClientGetInterfaceId(Id) Then
            If lvClientNetworks.SelectedItems.Count = 0 Then
                MessageBox.Show("Select network")
            Else
                If cbUseBssMac.Checked AndAlso lvClientBss.SelectedItems.Count = 0 Then
                    MessageBox.Show("Select Bss")
                Else
                    Dim Mac As String = ""
                    If cbUseBssMac.Checked Then
                        Mac = lvClientBss.SelectedItems(0).SubItems(2).Text
                    End If

                    Dim Mode As wclWiFi.wclWiFiConnectionMode
                    If cbClientDiscoverSecure.Checked Then
                        Mode = wclWiFi.wclWiFiConnectionMode.cmDiscoverySecure
                    Else
                        Mode = wclWiFi.wclWiFiConnectionMode.cmDiscoveryUnsecure
                    End If
                    ShowResult(Client.Connect(Id, Mode, "", lvClientNetworks.SelectedItems(0).SubItems(1).Text, _
                        ClientGetNetworkBss(), 0, Mac))
                End If
            End If
        End If
    End Sub

    Private Sub ClientConnectUsingSelectedprofile()
        Dim Id As Guid
        If ClientGetInterfaceId(Id) Then
            If lvClientNetworks.SelectedItems.Count = 0 Then
                MessageBox.Show("Select network")
            Else
                If lvClientProfiles.SelectedItems.Count = 0 Then
                    MessageBox.Show("Select profile")
                Else
                    If cbUseBssMac.Checked AndAlso lvClientBss.SelectedItems.Count = 0 Then
                        MessageBox.Show("Select Bss")
                    Else
                        Dim Mac As String = ""
                        If cbUseBssMac.Checked Then
                            Mac = lvClientBss.SelectedItems(0).SubItems(2).Text
                        End If

                        ShowResult(Client.Connect(Id, wclWiFi.wclWiFiConnectionMode.cmProfile, _
                            lvClientProfiles.SelectedItems(0).Text, "", ClientGetNetworkBss(), 0, Mac))
                    End If
                End If
            End If
        End If
    End Sub

    Private Sub btTurnOn_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btTurnOn.Click
        Dim Id As Guid
        If ClientGetInterfaceId(Id) Then
            ClientSwitchState(Id, False)
        End If
    End Sub

    Private Sub btTurnOff_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btTurnOff.Click
        Dim Id As Guid
        If ClientGetInterfaceId(Id) Then
            ClientSwitchState(Id, True)
        End If
    End Sub

    Private Sub ClientSwitchState(ByVal Id As Guid, ByVal Off As Boolean)
        Dim Iface As wclWiFi.wclWiFiInterface = New wclWiFi.wclWiFiInterface(Id)
        Try
            If ShowResult(Iface.Open()) Then
                Try
                    If Off Then
                        ShowResult(Iface.TurnOff())
                    Else
                        ShowResult(Iface.TurnOn())
                    End If
                Finally
                    Iface.Close()
                End Try
            End If
        Finally
            Iface = Nothing
        End Try
    End Sub

    Private Sub btCurrentIp_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btCurrentIp.Click
        Dim Id As Guid
        If ClientGetInterfaceId(Id) Then
            Dim Iface As wclWiFi.wclWiFiInterface = New wclWiFi.wclWiFiInterface(Id)
            If ShowResult(Iface.Open()) Then
                Dim [Static] As Boolean
                Dim Dns1 As String = ""
                Dim Dns2 As String = ""
                Dim Address As String = ""
                Dim Gateway As String = ""
                Dim Mask As String = ""
                Dim Res As Int32 = Iface.GetCurrentIp([Static], Address, Mask, Gateway, Dns1, Dns2)
                If ShowResult(Res) Then
                    Dim Msg As String
                    If [Static] Then
                        Msg = "Static IP" + vbCrLf
                    Else
                        Msg = "DHCP" + vbCrLf
                    End If
                    Msg = Msg + "Address: " + Address + vbCrLf + _
                        "Subnet mask: " + Mask + vbCrLf + _
                        "Default gateway: " + Gateway + vbCrLf + _
                        "Name server 1: " + Dns1 + vbCrLf + _
                        "Name server 2: " + Dns2
                    MessageBox.Show(Msg)
                    Iface.Close()
                End If
                Iface = Nothing
            End If
        End If
    End Sub

    Private Sub btGetIpSettings_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btGetIpSettings.Click
        Dim Id As Guid
        If ClientGetInterfaceId(Id) Then
            Dim Iface As wclWiFi.wclWiFiInterface = New wclWiFi.wclWiFiInterface(Id)
            If ShowResult(Iface.Open()) Then
                Dim [Static] As Boolean
                Dim Dns1 As String = ""
                Dim Dns2 As String = ""
                Dim Address As String = ""
                Dim Gateway As String = ""
                Dim Mask As String = ""
                Dim Res As Int32 = Iface.GetIpSettings([Static], Address, Mask, Gateway, Dns1, Dns2)
                If ShowResult(Res) Then
                    Dim Msg As String
                    If [Static] Then
                        Msg = "Static IP" + vbCrLf
                    Else
                        Msg = "DHCP" + vbCrLf
                    End If
                    Msg = Msg + "Address: " + Address + vbCrLf + _
                        "Subnet mask: " + Mask + vbCrLf + _
                        "Default gateway: " + Gateway + vbCrLf + _
                        "Name server 1: " + Dns1 + vbCrLf + _
                        "Name server 2: " + Dns2
                    MessageBox.Show(Msg)
                    Iface.Close()
                End If
                Iface = Nothing
            End If
        End If
    End Sub

    Private Sub btEnableDhcp_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btEnableDhcp.Click
        Dim Id As Guid
        If ClientGetInterfaceId(Id) Then
            Dim Iface As wclWiFi.wclWiFiInterface = New wclWiFi.wclWiFiInterface(Id)
            If ShowResult(Iface.Open()) Then
                ShowResult(Iface.EnableDhcp())
                Iface.Close()
            End If
            Iface = Nothing
        End If
    End Sub

    Private Sub btEnableStaticIp_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btEnableStaticIp.Click
        Dim Id As Guid
        If ClientGetInterfaceId(Id) Then
            Dim Iface As wclWiFi.wclWiFiInterface = New wclWiFi.wclWiFiInterface(Id)
            If ShowResult(Iface.Open()) Then
                ShowResult(Iface.SetStaticIp("192.168.1.210", "255.255.255.0", "192.168.1.1", "192.168.1.1", ""))
                Iface.Close()
            End If
            Iface = Nothing
        End If
    End Sub

    Private Sub btShowEditUi_Click(sender As System.Object, e As System.EventArgs) Handles btShowEditUi.Click
        Dim Id As Guid
        If ClientGetInterfaceId(Id) Then
            Dim Profile As String = ""
            If ClientGetProfile(Profile) Then
                ShowResult(ProfilesManager.ShowUIEdit(Id, Profile))
            End If
        End If
    End Sub

    Private Sub btSetXmlUserData_Click(sender As System.Object, e As System.EventArgs) Handles btSetXmlUserData.Click
        Dim Id As Guid = Guid.Empty
        If ClientGetInterfaceId(Id) Then
            Dim Profile As String = ""
            If ClientGetProfile(Profile) Then
                openFileDialog.FileName = ""
                If openFileDialog.ShowDialog() = DialogResult.OK Then
                    Dim Xml As String
                    Xml = System.IO.File.ReadAllText(openFileDialog.FileName)

                    ShowResult(ProfilesManager.SetProfileEapXmlUserData(Id, Profile, False, Xml))
                End If
            End If
        End If
    End Sub

    Private Sub btGetDualStaState_Click(sender As System.Object, e As System.EventArgs) Handles btGetDualStaState.Click
        Dim Id As Guid = Guid.Empty
        If ClientGetInterfaceId(Id) Then
            Dim Iface As wclWiFi.wclWiFiInterface = New wclWiFi.wclWiFiInterface(Id)
            If ShowResult(Iface.Open()) Then
                Dim State As Boolean
                If ShowResult(Iface.GetSecondarySta(State)) Then
                    MessageBox.Show("Dual-STA state: " + State.ToString())
                End If
                Iface.Close()
            End If
        End If
    End Sub

    Private Sub ClientSetDualState(Enable As Boolean)
        Dim Id As Guid = Guid.Empty
        If ClientGetInterfaceId(Id) Then
            Dim Iface As wclWiFi.wclWiFiInterface = New wclWiFi.wclWiFiInterface(Id)
            If ShowResult(Iface.Open()) Then
                ShowResult(Iface.SetSecondarySta(Enable))
                Iface.Close()
            End If
        End If
    End Sub

    Private Sub btEnableDualSta_Click(sender As System.Object, e As System.EventArgs) Handles btEnableDualSta.Click
        ClientSetDualState(True)
    End Sub

    Private Sub btDisableDualSta_Click(sender As System.Object, e As System.EventArgs) Handles btDisableDualSta.Click
        ClientSetDualState(False)
    End Sub

    Private Sub btEnumSecondaryInterfaces_Click(sender As System.Object, e As System.EventArgs) Handles btEnumSecondaryInterfaces.Click
        Dim Id As Guid
        If ClientGetInterfaceId(Id) Then
            Dim Iface As wclWiFi.wclWiFiInterface = New wclWiFi.wclWiFiInterface(Id)
            If ShowResult(Iface.Open()) Then
                Dim Ifaces() As wclWiFi.wclWiFiInterfaceData = Nothing
                If ShowResult(Iface.EnumInterfaces(Ifaces)) Then
                    ClientClearInterfaces()
                    If Ifaces IsNot Nothing Then
                        Dim i As Int32
                        For i = 0 To Ifaces.Length - 1
                            Dim Item As ListViewItem = lvIClientnterfaces.Items.Add(Ifaces(i).Id.ToString())
                            Item.SubItems.Add(Ifaces(i).Description)
                            Item.SubItems.Add(Ifaces(i).Primary.ToString())
                        Next i
                    End If
                End If
                Iface.Close()
            End If
        End If
    End Sub
End Class
